/**
 * CheapWAV.java: quick analysis of WAVE files.
 *            Copyright (C) 2012,2013 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-12-18 donfede@casagrau.org
 * credit to: Ringdroid, with a primary developer of Dominic Mazzoni, a Google employee and also author of Audacity. --  http://code.google.com/p/ringdroid/
 * $Id: CheapWAV.java,v 1.25 2013-04-02 02:35:24 donfede Exp $
 */

/*
 * ORIGINAL Ringdroid Apache/BSD Copyright by Google:
 * Copyright (C) 2008 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.ringdroid.soundfile;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

import android.util.Log;

/**
 * CheapWAV represents a standard 16-bit WAV file, splitting it into
 * artificial frames of 20 ms and taking the maximum of each frame to
 * get an approximation of the waveform contour.
 */
public class CheapWAV extends CheapSoundFile {
	private static final String LOGTAG = "CheapWAV";
	
	public static Factory getFactory() {
        return new Factory() {
            public CheapSoundFile create() {
                return new CheapWAV();
            }
            public String[] getSupportedExtensions() {
                return new String[] { "wav" };
            }
        };
    }

    // Member variables containing frame info
    private int mNumFrames;
    private int mNumNoiseFrames;
    private int[] mFrameOffsets = new int [0];
    private int[] mFrameLens = new int [0];
    private int[] mFrameGains = new int [0];
    private int[] mFrameNoise = new int[0];
    private int mFrameBytes;
    private int mFileSize;
    private int mSampleRate;
    private int mBitRate;
    private int mChannels;

    public CheapWAV() {
    }

    public int getNumFrames() {
        return mNumFrames;
    }
    
    public int getNumNoiseFrames() {
    	return mNumNoiseFrames;
    }

    public int getSamplesPerFrame() {
        return mSampleRate / 50;
    }

    public int[] getFrameOffsets() {
        return mFrameOffsets;
    }

    public int[] getFrameLens() {
        return mFrameLens;
    }

    public int[] getFrameGains() {
        return mFrameGains;
    }
    
    public int[] getFrameNoise() {
    	return mFrameNoise;
    }
    

    public int getFrameBytes() {
    	return mFrameBytes;
    }

    public int getFileSizeBytes() {
        return mFileSize;        
    }
    
    public long getFileSizeMS() {

    	int lframeSize = (mBitRate/8)*mChannels;
    	Log.d(LOGTAG, "getFileSizeMS() 22  lframeSize:" + lframeSize + " mFrameBytes: " + mFrameBytes );
    	Log.d(LOGTAG, "getFileSizeMS() 23  mFileSize:" + mFileSize);
    	long lframeCount = 0;
    	if (0 != lframeSize) {
        	lframeCount = mFileSize / lframeSize;    		
    	}
    	Log.d(LOGTAG, "getFileSizeMS() 24  lframeCount:" + lframeCount + " mNumFrames: " + mNumFrames);
		int lfrequency = mSampleRate; // cycles per second
    	double lfrequencyMS = lfrequency / 1000.0; // cycles per millisecond
    	Log.d(LOGTAG, "getFileSizeMS() 25  mSampleRate:" + mSampleRate + " lfrequency: " + lfrequency + " lfrequencyMS: " + lfrequencyMS);

    	
    	long lfileMS = 0;
    	if (0 != lfrequencyMS) {
    		lfileMS = (long) (lframeCount / lfrequencyMS);
    		Log.d(LOGTAG, "getFileSizeMS() 98  fileMS: " + lfileMS  + " fileMS w/o50: " + (mNumFrames/mSampleRate) );
    	}
    	Log.d(LOGTAG, "getFileSizeMS() 99  numframe:" + mNumFrames + " samplerate: " + mSampleRate + " sR/50: " + (mSampleRate/50) + " fileMS: " + lfileMS );
    	return lfileMS;
    } // END getFileSizeMS()

    public int getSampleRate() {
        return mSampleRate;
    }

    public int getChannels() {
        return mChannels;
    }
    
    public int getBitRate() {
    	return mBitRate;
    }

    public String getFiletype() {
        return "WAV";
    }

    public void ReadFile(File inputFile)
            throws java.io.FileNotFoundException,
                   java.io.IOException {
        super.ReadFile(inputFile); 
    	Log.d(LOGTAG, "ReadFile() 00  inputFile: " + inputFile);

        // Try to read meta data from a cached file.
        boolean lrc = ReadFileMeta(inputFile);
        if (lrc) return;
        
    	Log.d(LOGTAG, "ReadFile() 99");
    } // END ReadFile()


    /**
     * Read the state of key data members from disk.
     * @param inputParentFile File of the parent audio file.
     * @return boolean flag, true if data was read from disk.
     * This method works in conjunction with {@link #WriteFileMeta()}.
     */
    @SuppressWarnings({ "deprecation", "unused" })
	public boolean ReadFileMeta(File inputParentFile) {
    	Log.d(LOGTAG, "ReadFileMeta() 00  P File: " + inputParentFile.toString());

    	boolean lrc = false;	// Return code.

    	String lparentFileName = inputParentFile.getName();
    	String[] lcomponents = lparentFileName.split("\\.");
        if (lcomponents.length < 2) {
        	Log.e(LOGTAG, "ReadFileMeta() 15 -> 99  EXIT ignoring error, insufficient components: " + lcomponents.toString());
            return lrc;
        }
        String lmetaFileName = inputParentFile.getParent() + File.separator + "meta_dir" + File.separator + lcomponents[lcomponents.length-2] + ".meta";
    	File lMetaFile = new File(lmetaFileName);
    	long lMetaLen = lMetaFile.length();
        Log.d(LOGTAG, "ReadFileMeta() 22   M File: " + lMetaFile + " MetaLen: " + lMetaLen);

        // Check that meta file is valid.
        if ( (!lMetaFile.isFile()) || (0 == lMetaLen) ) {
        	Log.w(LOGTAG, "ReadFileMeta() 25 -> 99  EXIT ignoring error, invalid meta file");
        	return lrc;
        }
        
        //Log.d(LOGTAG, "ReadFileMeta() 30  original object: " + this.toString() );
        String lheader = "";
        String ldatestamp = "";
        String lreadFileName = "";
        int lmetaVersion = 0;
        // Credit to IBM page on java serializing --  http://www.ibm.com/developerworks/java/library/j-5things1/index.html
        try {
	        FileInputStream fis = new FileInputStream(lMetaFile);
	        ObjectInputStream ois = new ObjectInputStream(fis);
	        lheader = ois.readLine();
	        ldatestamp = ois.readLine();
	        lmetaVersion = ois.readInt();
	    	Log.d(LOGTAG, "ReadFileMeta() 31  metaVersion: " + lmetaVersion);

	        if ( (3 == lmetaVersion) ||
	        		(2 == lmetaVersion) ) {
		        lreadFileName = (String) ois.readObject();
		        mFileSize = ois.readInt();
		        mChannels = ois.readInt();
		        mSampleRate = ois.readInt();
		        mBitRate = ois.readInt();
		        mNumFrames = ois.readInt();
		        mNumNoiseFrames = ois.readInt();
		        if (3 == lmetaVersion)
		        	mFrameBytes = ois.readInt();
		        // FIXME: must find a way to update older meta files. ... possibly during DB upgrade (not sure if that's in a separate thread).
                if (mProgressListener != null) {	// FIXME: this listener check sure feels like a function...
                    boolean keepGoing = mProgressListener.reportProgress(0.10);
                    if (!keepGoing) {
                        return lrc;
                    }
                }
		        mFrameOffsets = (int[]) ois.readObject();
		        if (mProgressListener != null) {
                    boolean keepGoing = mProgressListener.reportProgress(0.25);
                    if (!keepGoing) {
                        return lrc;
                    }
                }
		        mFrameLens = (int[]) ois.readObject();
                if (mProgressListener != null) {
                    boolean keepGoing = mProgressListener.reportProgress(0.50);
                    if (!keepGoing) {
                        return lrc;
                    }
                }
		        mFrameGains = (int[]) ois.readObject();
                if (mProgressListener != null) {
                    boolean keepGoing = mProgressListener.reportProgress(0.75);
                    if (!keepGoing) {
                        return lrc;
                    }
                }
		        mFrameNoise = (int[]) ois.readObject();
                if (mProgressListener != null) {
                    boolean keepGoing = mProgressListener.reportProgress(0.98);
                    if (!keepGoing) {
                        return lrc;
                    }
                }
	        } else {
	        	Log.w(LOGTAG, "ReadFileMeta() 37 -> 99  EXIT ignoring error, invalid meta file version: " + lmetaVersion);
	        	return lrc;
	        }
	        ois.close();
	        lrc = true;
        } catch (Exception e) {
            Log.e(LOGTAG, "ReadFileMeta() Faile to read meta file: " + lMetaFile.toString() + " exception ex: " + e.getLocalizedMessage() );
            e.printStackTrace();
        }        
//        Log.d(LOGTAG, "ReadFileMeta() 41 header: " + lheader);
//        Log.d(LOGTAG, "ReadFileMeta() 43 datestamp: " + ldatestamp);
//        Log.d(LOGTAG, "ReadFileMeta() 44 version: " + lmetaVersion);
//        Log.d(LOGTAG, "ReadFileMeta() 45 read filename: " + lreadFileName);
//        Log.d(LOGTAG, "ReadFileMeta() 47  read object: " + this.toString() );

        return lrc;
    } // END WriteFileMeta(File inputParentFile)
    
    
    
    /**
     * Return a string representation of this object.
     * @return String with description of object contents.
     */
    public String toString() {
        String lstr = new String();    
    	lstr = lstr + " mFileSize: " + mFileSize;
    	lstr = lstr + " mChannels: " + mChannels;
    	lstr = lstr + " mSampleRate: " + mSampleRate;
    	lstr = lstr + " mBitRate: " + mBitRate;
    	lstr = lstr + " mNumFrames: " + mNumFrames;
    	lstr = lstr + " nNumNoiseFrames: " + mNumNoiseFrames;
    	lstr = lstr + " mFrameOffsets[" + mFrameOffsets.length + "]: " + mFrameOffsets.toString();
    	lstr = lstr + " mFrameLens[" + mFrameLens.length + "]: " + mFrameLens.toString();
    	lstr = lstr + " mFrameGains[" + mFrameGains.length + "]: " + mFrameGains.toString();
    	lstr = lstr + " mFrameNoise[" + mFrameNoise.length + "]: " + mFrameNoise.toString();
    	return lstr;
    } // END toString()
    
} // END class CheapWAV

